@@ -190,7 +190,8 @@ static int wp_tdmv_tx_dchan(struct zt_chan *chan, int len);
 #endif
 static int wp_tdmv_rx_dchan(wan_tdmv_t*, int, unsigned char*, unsigned int); 
 
-static int wp_tdmv_span_buf_rotate(void *pcard, u32, unsigned long);
+static int wp_tdmv_span_buf_rotate(void *pcard, u32, unsigned long, int);
+
 
 static void wp_tdmv_report_alarms(void* pcard, unsigned long te_alarm);
 
@@ -273,13 +274,17 @@ wp_bri_zap_ioctl(struct zt_chan *chan, unsigned int cmd, unsigned long data)
 static int wp_bri_zap_open(struct zt_chan *chan)
 {
 	wp_tdmv_bri_t	*wr = NULL;
+	sdla_t *card;
 	
 	BRI_FUNC();
 
 	WAN_ASSERT2(chan == NULL, -ENODEV);
 	WAN_ASSERT2(chan->pvt == NULL, -ENODEV);
 	wr = chan->pvt;
+	WAN_ASSERT2(wr->card == NULL, -ENODEV);
+    card = wr->card;
 	wr->usecount++;
+	wanpipe_open(card);
 	wan_set_bit(WP_TDMV_RUNNING, &wr->flags);
 	DEBUG_EVENT("%s: Open (usecount=%d, channo=%d, chanpos=%d)...\n", 
 				wr->devname,
@@ -300,13 +305,15 @@ static int wp_bri_zap_close(struct zt_chan *chan)
 	WAN_ASSERT2(chan == NULL, -ENODEV);
 	WAN_ASSERT2(chan->pvt == NULL, -ENODEV);
 	wr	= chan->pvt;
+	WAN_ASSERT2(wr->card == NULL, -ENODEV);
 	card	= wr->card;
+	wanpipe_close(card);
 	fe	= &card->fe;
 	wr->usecount--;
 	wan_clear_bit(WP_TDMV_RUNNING, &wr->flags);
 
 	return 0;
-}
+}           
 
 static int wp_bri_zap_watchdog(struct zt_span *span, int event)
 {
@@ -331,7 +338,6 @@ static int wp_tdmv_bri_hwec_create(struct dahdi_chan *chan, struct dahdi_echocan
 {
 	wp_tdmv_bri_t *wr = NULL;
 	sdla_t *card = NULL;
-	int	fe_chan = chan->chanpos;
 	int	err = -ENODEV;
 
 	if (ecp->param_count > 0) {
@@ -346,19 +352,19 @@ static int wp_tdmv_bri_hwec_create(struct dahdi_chan *chan, struct dahdi_echocan
 	WAN_ASSERT2(wr->card == NULL, -ENODEV);
 	card = wr->card;
 
-	*ec = &wr->ec[fe_chan];
+	*ec = &wr->ec[chan->chanpos-1];
 	(*ec)->ops = &wp_tdmv_bri_ec_ops;
 	(*ec)->features = wp_tdmv_bri_ec_features;
 
 	if (card->wandev.ec_enable) {
 		DEBUG_EVENT("[TDMV_BRI]: %s: %s(): channel %d\n",
-			wr->devname, __FUNCTION__, fe_chan);
+			wr->devname, __FUNCTION__, chan->chanpos);
 
-		if(fe_chan == 1 || fe_chan == 2) {
-			err = card->wandev.ec_enable(card, 1, fe_chan);
+		if(chan->chanpos == 1 || chan->chanpos == 2) {
+			err = card->wandev.ec_enable(card, 1, chan->chanpos);
 		} else {
 			DEBUG_EVENT("[TDMV_BRI]: %s: %s(): Warning: invalid fe_channel %d!!\n",
-				wr->devname, __FUNCTION__, fe_chan);
+				wr->devname, __FUNCTION__, chan->chanpos);
 			err = 0;
 		}
 	} else {
@@ -377,7 +383,6 @@ static void wp_tdmv_bri_hwec_free(struct dahdi_chan *chan, struct dahdi_echocan_
 {
 	wp_tdmv_bri_t	*wr = NULL;
 	sdla_t *card = NULL;
-	int	fe_chan = chan->chanpos;
 
 	memset(ec, 0, sizeof(*ec));
 
@@ -389,13 +394,13 @@ static void wp_tdmv_bri_hwec_free(struct dahdi_chan *chan, struct dahdi_echocan_
 
 	if (card->wandev.ec_enable) {
 		DEBUG_EVENT("[TDMV_BRI]: %s: %s(): channel %d\n",
-			wr->devname, __FUNCTION__, fe_chan);
+			wr->devname, __FUNCTION__, chan->chanpos);
 
-		if(fe_chan == 1 || fe_chan == 2) {
-			card->wandev.ec_enable(card, 0, fe_chan);
+		if(chan->chanpos == 1 || chan->chanpos == 2) {
+			card->wandev.ec_enable(card, 0, chan->chanpos);
 		} else {
 			DEBUG_EVENT("[TDMV_BRI]: %s: %s(): Warning: invalid fe_channel %d!!\n",
-				wr->devname, __FUNCTION__, fe_chan);
+				wr->devname, __FUNCTION__, chan->chanpos);
 		}
 	} else {
 		DEBUG_EVENT("[TDMV_BRI]: %s: %s(): card->wandev.ec_enable == NULL!!!!!!\n",
@@ -413,10 +418,9 @@ static void wp_tdmv_bri_hwec_free(struct dahdi_chan *chan, struct dahdi_echocan_
 static int wp_bri_zap_hwec(struct zt_chan *chan, int enable)
 {
 	wp_tdmv_bri_t   *wr = NULL;
-	sdla_t      *card = NULL;
-	int     fe_chan = chan->chanpos;
-	int     err = -ENODEV;
-	sdla_fe_t   *fe = NULL;
+	sdla_t      	*card = NULL;
+	int     		err = -ENODEV;
+	sdla_fe_t  	 	*fe = NULL;
 
 	BRI_FUNC();
 
@@ -429,13 +433,13 @@ static int wp_bri_zap_hwec(struct zt_chan *chan, int enable)
 
 	if (card->wandev.ec_enable){
 		DEBUG_EVENT("[TDMV_BRI]: %s: %s(): channel %d\n",
-					wr->devname, __FUNCTION__, fe_chan);
+					wr->devname, __FUNCTION__, chan->chanpos);
 
-		if(fe_chan == 1 || fe_chan == 2){
-			err = card->wandev.ec_enable(card, enable, fe_chan);
+		if(chan->chanpos == 1 || chan->chanpos == 2){
+			err = card->wandev.ec_enable(card, enable, chan->chanpos);
 		}else{
 			DEBUG_EVENT("[TDMV_BRI]: %s: %s(): Warning: invalid fe_channel %d!!\n",
-						wr->devname, __FUNCTION__, fe_chan);
+						wr->devname, __FUNCTION__, chan->chanpos);
 			err = 0;
 		}
 	}else{
@@ -982,7 +986,9 @@ static int wp_tdmv_bri_remove(void* pcard)
 	return 0;
 }
 
-static int wp_tdmv_span_buf_rotate(void *pcard, u32 buf_sz, unsigned long mask)
+
+
+static int wp_tdmv_span_buf_rotate(void *pcard, u32 buf_sz, unsigned long mask, int circ_buf_len)
 {
 	sdla_t		*card = (sdla_t*)pcard;
 	wan_tdmv_t	*wan_tdmv = &card->wan_tdmv;
@@ -990,18 +996,28 @@ static int wp_tdmv_span_buf_rotate(void *pcard, u32 buf_sz, unsigned long mask)
 	int x;
 	unsigned int rx_offset, tx_offset;
 	void *ptr;
-	
+
 	WAN_ASSERT(wan_tdmv->sc == NULL);
 	wp = wan_tdmv->sc; 
 
-	rx_offset = buf_sz * card->u.aft.tdm_rx_dma_toggle;
-	tx_offset = buf_sz * card->u.aft.tdm_tx_dma_toggle;
-
 #define BRI_NUM_OF_BCHANNELS 2
 	for (x = 0; x < BRI_NUM_OF_BCHANNELS; x ++) {
 	/*for (x = 0; x < 32; x ++) {*/
 		if (wan_test_bit(x,&wp->timeslot_map)) {
 
+			rx_offset = buf_sz * card->u.aft.tdm_rx_dma_toggle[x];
+			tx_offset = buf_sz * card->u.aft.tdm_tx_dma_toggle[x];	
+
+			card->u.aft.tdm_rx_dma_toggle[x]++;
+			if (card->u.aft.tdm_rx_dma_toggle[x] >= circ_buf_len) {
+				card->u.aft.tdm_rx_dma_toggle[x]=0;
+			}
+			card->u.aft.tdm_tx_dma_toggle[x]++;
+			if (card->u.aft.tdm_tx_dma_toggle[x] >= circ_buf_len) {
+				card->u.aft.tdm_tx_dma_toggle[x]=0;
+			}
+
+
 			wan_spin_lock(&wp->chans[x].lock);
 				
                         ptr=(void*)((((unsigned long)wp->chans[x].readchunk) & ~(mask)) + rx_offset);
