@@ -421,7 +421,11 @@ int wanpipe_tdm_api_reg(wanpipe_tdm_api_dev_t *tdm_api)
 		tdm_api->tx_q_len		= WP_TDM_MAX_TX_Q_LEN;
 	
 		if (tdm_api->cfg.idle_flag == 0) {
-        		tdm_api->cfg.idle_flag=0xFF; 	
+			if (tdm_api->cfg.hw_tdm_coding == WP_MULAW) {
+				tdm_api->cfg.idle_flag=0xFF;
+			} else {
+				tdm_api->cfg.idle_flag=0xD5;	
+			}
 		} 
 	}
 	
