@@ -80,6 +80,22 @@ static int create_udp_socket(call_signal_connection_t *mcon, char *local_ip, int
 				   (char *)&flag, sizeof(int));
 #endif
 
+		
+			rc = fcntl(mcon->socket, F_GETFL);
+			if ( rc < 0 ) {
+				close(mcon->socket);
+				mcon->socket = -1;
+				return mcon->socket;
+			}
+			
+			rc |= O_NONBLOCK;
+			rc = fcntl(mcon->socket, F_SETFL, rc);
+			if ( rc < 0 ) {
+				close(mcon->socket);
+				mcon->socket = -1;
+				return mcon->socket;
+			}
+
 			if ((rc = bind(mcon->socket, 
 				  (struct sockaddr *) &mcon->local_addr,
 				   sizeof(mcon->local_addr))) < 0) {
@@ -90,7 +106,7 @@ static int create_udp_socket(call_signal_connection_t *mcon, char *local_ip, int
 				rc=listen(mcon->socket,100);
 				if (rc) {
 					close(mcon->socket);
-	                                mcon->socket = -1;
+					mcon->socket = -1;
 				}
 #endif
 			}
@@ -352,11 +368,10 @@ int call_signal_connection_writep(call_signal_connection_t *mcon, call_signal_ev
 	}
 #endif
 
-	pthread_mutex_lock(&mcon->lock);
 	event->version=SIGBOOST_VERSION;
+	/* Sendto is thread safe so no lock needed */
 	err=sendto(mcon->socket, event, event_size, 0, 
 		   (struct sockaddr *) &mcon->remote_addr, sizeof(mcon->remote_addr));
-	pthread_mutex_unlock(&mcon->lock);
 
 	if (err != event_size) {
 		err = -1;
@@ -424,8 +439,13 @@ int call_signal_connection_write(call_signal_connection_t *mcon, call_signal_eve
 
 	event->bseqno=mcon->rxseq;
 	event->version=SIGBOOST_VERSION;
+	
 	err=sendto(mcon->socket, event, event_size, 0, 
 		   (struct sockaddr *) &mcon->remote_addr, sizeof(mcon->remote_addr));
+	
+	if (err != event_size) {
+		mcon->txseq--;	
+	}
 	pthread_mutex_unlock(&mcon->lock);
 
 	if (err != event_size) {
